program ECS_Change_FSB;

{$S-}
{$I-}
type
  str8 = string[8];
  str6 = string[6];
  rec = record
     speed : str8;
     v1, v2, v3 : byte;
  end;

function Hex(b: byte) : string;
const hexChars: array [0..$F] of Char ='0123456789ABCDEF';
begin
  hex:=hexChars[b shr 4] + hexChars[b and $F];
end;

function ChBits(v : byte; mask : string) : byte;
var i : byte;
begin
  for i:=0 to 7 do
    case mask[i+1] of
     '0' : v:=v and not($80 shr i);
     '1' : v:=v or ($80 shr i);
    end;
  ChBits:=v;
end;

procedure ChBitsL(var r1, r2 : byte; s : string; i : byte; c : char);
var
  w : word;
  m1, m2 : word;
  p, n : byte;
begin
  w:= r1*$100 + r2;
  p:= 15 - pos(c, s);

  n:= 0;
  while Pos(c, s) > 0 do begin
    s[Pos(c, s)] := '1';
    inc(n);
  end;

  i:= i shl (8-n);
  i:= i shr (8-n);
  m1:= not(0) shl n;
  m2:= i;
  asm
    mov cl, p
    rol m1,cl
    rol m2,cl
  end;
  w:= w and m1;
  w:= w or m2;

  r1:=hi(w);
  r2:=lo(w);
end;

function CutRet(var s : string) : string;
var i : byte;
begin
  i:=pos(' ', s);
  if i=0 then begin
    CutRet:=s;
    s:='';
    exit;
  end;
  CutRet:=copy(s, 1, i-1);
  delete(s, 1, i);
end;

function ReadCmos(position : byte) : byte;
begin
  port[$70]:=position;
  ReadCmos:=port[$71];
end;

procedure WriteCmos(position, value : byte);
begin
  port[$70]:=position;
  port[$71]:=value;
end;

function BiosVer : str6;
var v : array[1..8] of char absolute $F000:$FFF5;
begin
  BiosVer:=v[7]+v[8]+v[1]+v[2]+v[4]+v[5];
end;

function IsShift : boolean;
begin
  IsShift:= mem[$0:$0417] and 3<>0;
end;

procedure quit(s : string);
begin
  writeln(s);
  halt(0);
end;

procedure DumpCmos;
var i : byte;
begin
  writeln(#10'CMOS dump');
  write('     ');
  for i:=0 to $f do write(Hex(i), ' ');
  write(#13#10'----------------------------------------------------');
  for i:=0 to 127 do begin
    if (i mod $10)=0 then write(#13#10,Hex(i),' - ');
    write(Hex(ReadCmos(i)), ' ');
  end;
  writeln;
end;


function h2d(s : string) : byte;
var v, code : integer;
begin
  val('$'+s, v, code);
  h2d:=v;
end;

procedure DumpBios;
var
  f : file;
  bios1 : array[0..$7fff] of byte absolute $F000:0;
  bios2 : array[0..$7fff] of byte absolute $F000:$8000;
begin
  assign(f, 'bios.bin'); rewrite(f, 1);
  blockwrite(f, bios1, sizeof(bios1));
  blockwrite(f, bios2, sizeof(bios2));
  close(f);
end;

{------------------------------------------------------------------------}
var
  set1 : array[1..9] of rec;
  i : integer;
  csum : word;
  t : text;
  bver, sp, path : string;
  r70, r71, r72 : byte;
  r57, r58 : byte;
  opc : string;
begin
  writeln('FSB tweak v0.15 by mpapec@yahoo.com and Cheepoman(for ECS K7S5A boards only)');
  writeln('Greetings to cbozo and others at OCworkbench forum!');
  writeln('CHFSB is released under GPL, for additional info read gpl.txt');
  bver:=BiosVer;
  writeln(#10'Bios version: ', bver);

  path:=paramstr(0); dec(byte(path[0]),9);

  if (paramstr(1)='/d') then begin
    DumpCmos;
    halt(0);
  end;

  if (paramstr(1)='/b') then begin
    DumpBios;
    halt(0);
  end;

  if (paramstr(1)='/m') then begin
    assign(t, path+'memall.txt'); reset(t);
    if IOResult<>0 then quit('memall.txt is missing!');

    sp:='';
    while not(eof(t)) and (pos(bver, sp)=0) do readln(t, sp);
    if eof(t) then quit('No luck, your BIOS is not on my list.');
    writeln('Picking: ', sp);
    readln(t, sp);

    r57:=ReadCmos($57);
    r58:=ReadCmos($58);
    csum:=ReadCmos($3e)*$100+ReadCmos($3f);
    csum:=csum-r57-r58;

  {
    RAS Precharge time(0=3T, 2=4T, 1=2T)
    RAS Active time(0=6T, 1=7T, 2=5T, 3=4T)
    CAS Latency(0=SPD, 1=2T, 2=2.5T, 3=3T)
    DRAM Timing(Configuration 0=SAFE, 1=NORMAL, 2=FAST, 3=ULTRA)
  }

    write('RAS Precharge time(2, 3, 4): '); readln(i);
    case i of
      2 : i:=1;
      3 : i:=0;
      4 : i:=2;
    end;
    ChBitsL(r57, r58, sp, i, 'A');

    write('RAS Active time(4, 5, 6, 7): '); readln(i);
    case i of
      4 : i:=3;
      5 : i:=2;
      6 : i:=0;
      7 : i:=1;
    end;
    ChBitsL(r57, r58, sp, i, 'B');

    write('CAS Latency(0=SPD, 2, 2.5, 3): '); readln(opc);
    if opc='0' then i:=0
    else if opc='2' then i:=1
    else if opc='2.5' then i:=2
    else if opc='3' then i:=3;
    ChBitsL(r57, r58, sp, i, 'C');

    write('DRAM Timing(Configuration 0=SAFE, 1=NORMAL, 2=FAST, 3=ULTRA): '); readln(i);
    ChBitsL(r57, r58, sp, i, 'D');

    csum:=csum+r57+r58;
    WriteCmos($57, r57);
    WriteCmos($58, r58);
    WriteCmos($3e, hi(csum));
    WriteCmos($3f, lo(csum));
    write('Press [Enter] to continue or [ctrl]+[alt]+[del] to reboot');
    readln;
    halt(0);
  end;


  if (paramstr(1)<>'/i') and not(IsShift) then begin
    writeln(#10#13'  /i - run FSB settings');
    writeln('  /m - memory settings(expert mode!)');
    writeln('  /d - dump cmos memory');
    writeln('  /b - dump BIOS at 0xF000');
    halt(0);
  end;

  assign(t, path+'fsball.txt'); reset(t);
  if IOResult<>0 then quit('fsball.txt is missing!');

  sp:='';
  while not(eof(t)) and (pos(bver, sp)=0) do readln(t, sp);
  if eof(t) then quit('No luck, your BIOS is not on my list.');
  writeln('Picking: ', sp);

  r70:=ReadCmos($70);
  r71:=ReadCmos($71);
  r72:=ReadCmos($72);
  for i:=1 to 9 do
    with set1[i] do begin
      readln(t, sp);
      speed:=CutRet(sp);
      v1:=ChBits(r70, CutRet(sp));
      v2:=ChBits(r71, CutRet(sp));
      if sp='' then sp:='xxxxxxxx';
      v3:=ChBits(r72, CutRet(sp));
    end;
  close(t);

  for i:=1 to 9 do
    with set1[i] do begin
      write(i, '. ', speed);
      if (r70=v1) and (r71=v2) and (r72=v3) then writeln(' <-') else writeln;
    end;

  write('Set FSB(0=none): '); readln(i);
  if i=0 then quit('Exiting..');

  with set1[i] do begin
    writeln('Seting FSB to ', speed);
    csum:=ReadCmos($3e)*$100+ReadCmos($3f);
    csum:=csum-r70-r71-r72;
    csum:=csum+v1+v2+v3;
    WriteCmos($70, v1);
    WriteCmos($71, v2);
    WriteCmos($72, v3);
    WriteCmos($3e, hi(csum));
    WriteCmos($3f, lo(csum));
    if ReadCmos($70)<>v1 then quit('Terminated, Win2k/XP users are advised to use DOS/win9x startup floppy disk!');
  end;
  write('Press [Enter] to continue or [ctrl]+[alt]+[del] to reboot');
  readln;
end.
